/**
 * @class
 */
communote.widget.classes.Initializer = communote.Base.extend(
/** 
 * @lends communote.widget.classes.Initializer.prototype
 */	
{
    ready: false,

    constructor: function(configuration) {
        var controllers;
        var self = this;
        var apiAccessor = new communote.widget.classes.CommunoteApiAccessor(configuration);
        var url = apiAccessor.buildApiInformationUrl();
        communote.jQuery.ajax({
            async: false,
            type: "get",
            url: url,
            cache: true,
            data: {},
            dataType: "json",
            success: function(data, textStatus) {
                if (data.userId) {
                    communote.currentUser = {
                        alias: data.userAlias,
                        firstName: data.userFirstName,
                        lastName: data.userLastName,
                        userId: data.userId
                    };
                }
            },
            error: function(data, textStatus, thrownError) {
                // TODO should render some error message but there is currently no way to do this
                throw new Error(thrownError);
            }
        });
        controllers = [ {
            object: new communote.widget.classes.controllers.I18nController(configuration, apiAccessor)
        }, {
            object: new communote.widget.classes.controllers.EventController()
        }, {
            object: new communote.widget.classes.controllers.TemplateController()
        }, {
            object: new communote.widget.classes.controllers.WidgetController()
        }, {
            object: new communote.widget.classes.controllers.ApiController(configuration, apiAccessor)
        }, {
            object: new communote.widget.classes.controllers.ControlFactory()
        } ];
        this.controllerCheckList = controllers;
        this.controller = {};
        // TODO pretty ugly coding style
        communote.jQuery.each(controllers, function(index, controller) {
            var ctrl = controller.object;
            controller.ready = false;
            self.controller[ctrl.name] = ctrl;
            ctrl.controller = self.controller;
            ctrl.init(self, configuration);
        });
    },

    /**
     * Callback for a controller to signal that there initialization is ready. 
     */
    controllerReady: function(readyController) {
        var i, controllerDescr;
        var ready = true;
        var controllers = this.controllerCheckList;

        for (i = 0; i < controllers.length; i++) {
            controllerDescr = controllers[i];
            if (readyController === controllerDescr.object) {
                controllerDescr.ready = true;
                break;
            }
        }
        for (i = 0; i < controllers.length; i++) {
            if (!controllers[i].ready) {
                ready = false;
                break;
            }
        }
        if (ready) {
            this.ready = true;
            this.startUp();
        }
    },

    initializeWidget: function(rootSelector, configuration) {
        var widget = communote.widget.WidgetController.createWidget(communote.jQuery(rootSelector), configuration);
        if (this.ready) {
            communote.widget.WidgetController.startWidget(widget);
        } else {
            this.queueWidget(widget);
        }
    },
    
    /**
     * 
     */
    queueWidget: function(widget) {
        var self = communote.widget.MainController;
        if (self.widgetQueue === undefined) {
            self.widgetQueue = [];
        }
        self.widgetQueue.push(widget);
    },

    /**
     * 
     */
    startUp: function() {
        if(this.widgetQueue) {
            communote.widget.WidgetController.addWidgets(this.widgetQueue);
        }
        this.widgetQueue = [];
        communote.widget.WidgetController.startWidgets();
    }
});

(function() {
    // singleton initializer
    var initializer = null;

    // Create a configuration by merging the default configuration with the provided object
    var getConfiguration = function(extend) {
        var key, newConfig, valueExtend, minValue, maxValue;
        // TODO won't work with more widgets on a page (object members are shared). Configuration class isn't useful anyway.
        newConfig = new communote.widget.classes.Configuration();
        if (extend) {
            for (key in extend) {
                // only copy defined values
                valueExtend = extend[key];
                if (extend.hasOwnProperty(key) && (valueExtend != undefined)) {
                    if (typeof valueExtend === 'number') {
                        // check min max boundaries if defined
                        minValue = newConfig[key + '_min'];
                        maxValue = newConfig[key + '_max'];
                        if (((minValue == undefined) || (valueExtend >= minValue)) && 
                                ((maxValue == undefined) || (valueExtend <= maxValue))) {
                            newConfig[key] = valueExtend;
                        }
                    } else {
                        newConfig[key] = valueExtend;
                    }
                }
            }
        }
        return newConfig;
    };
    
    // expose start function
    communote.widget.start = function(rootSelector, configuration) {

        communote.jQuery(document).ready(function() {        
            configuration = getConfiguration(configuration);
            if (!initializer) {
                initializer = new communote.widget.classes.Initializer(configuration);
            }
            initializer.initializeWidget(rootSelector, configuration);
        });
    };
})();